/****************************************************************/
/*								*/
/*	makefont						*/
/*								*/
/*	Initial version November, 1987				*/
/*	Updated March, 1988 -- added support for 300 		*/
/*		dot/inch printers				*/
/*      Updated August, 1989 -- Adapted to AIX Version 3        */
/*								*/
/*	A program to simplify the use of METAFONT 		*/
/*								*/
/*	This program, by Rick Simpson of IBM Austin, Texas,    	*/
/*	USA, is not copyrighted and can be used for any 	*/
/*	purpose. You are encouraged to copy "makefont.c" to 	*/
/*	your own directory and tailor it to suit your own 	*/
/*	circumstances.						*/
/*								*/
/*	Invocation:						*/
/*	makefont [-b base] [-l lib] [-v] [-d device]... 	*/
/*		   font [mag] 					*/
/*								*/
/*	Where:							*/
/*	    -b base   Base for METAFONT to use; default "cm".	*/
/*          -l lib    Specifies target library for font, tfm.   */
/*		      Default is current directory.		*/
/*	    -v 	      "Verbose" flag.  If specified, keep all	*/ 
/*		      miscellaneous output (".log").  If not	*/
/*		      specified, delete such.			*/
/*	    -d device Specifies a device name (e.g., 4216)	*/
/*		      There can be 0 or more of these names,	*/
/*		      each with its own "-d" flag.  If no 	*/
/*		      devices are specified, all unique 	*/
/*		      subdirectories in /usr/lpp/tex/fonts 	*/
/*		      will be used as names. 			*/
/*	    font      Required; name of font, WITHOUT ".mf"     */
/*	    mag	      Optional; default is no magnification.    */
/*								*/
/****************************************************************/
#include	<stdio.h>
#ifndef IBMRT
#include	<stdlib.h>
#else
double atof();
#endif
#include	<string.h>
#include	<ctype.h>
#include	<math.h>
#include	"devices.h"
#ifndef TRUE
#define		TRUE	(1)
#define		FALSE	(0)
#endif


#define if_debug_show_rc  if (debug) {\
      printf("    %s : rc=0x%X\n",command,rc);	\
      perror(""); }

/****************************************************************/
/*  								*/
/*  Global variables						*/
/*  								*/
/****************************************************************/

extern   device  *lookup_device();

device   *dev_anchor;		/* anchor of chain of dev blks	*/

#define  MAX_DEV  32

char	*device_name[MAX_DEV];	/* Names of devices		*/
int	 num_devices;		/* No. used in device_name[]	*/

char	*output_library;	/* Name of library		*/

char	*mag_parm;		/* Command-line mag parm	*/

char	*font_parm;		/* Command-line font name	*/

char	verbose;		/* TRUE if verbose output	*/

char    *mag_string;		/* mag string, for "mf" command	*/

double  mag_value;		/* mag value, as a float	*/

char    *base_parm;		/* Base name; default "cm"	*/

char    debug;			/* TRUE if debugging		*/
 

/****************************************************************/
/*								*/
/*  Main routine						*/
/*								*/
/****************************************************************/
 
main(argc,argv)
int	 argc;
char	*argv[];
{
    int   i;
    int   rc;

/*  Process command-line parameters				*/
    if (!init(argc,argv)) exit(1);	

/*  Parse the "devices" file to get device characteristics	*/
    if (!parse_devices()) {
	fprintf(stderr,
	    "Can't locate \"devices\" file.\n");
	exit(1);
    }

/*  Make sure the devices specified are known			*/
    for (i=0; i<num_devices; i++) 
	if (NULL == lookup_device(device_name[i])) {
	    fprintf
		(stderr,
	         "Device \"%s\" unknown.\n",
	         device_name[i]);
	    exit(1);
	}

/*  Check the specified magnification, if any			*/
    if (!check_mag()) exit(1);
		
/*  Everything looks OK;  call run_metafont() for each device	*/
    for (i=0; i<num_devices; i++) 
	run_metafont(lookup_device(device_name[i]));

    return(0);
}

/****************************************************************/
/*								*/
/*	run_metafont -- Execute METAFONT and GFtoPK for one 	*/
/*			font for one device.			*/
/*								*/
/*	If the output font already exists, don't bother to run.	*/
/*								*/
/****************************************************************/
 
run_metafont(p)
device  *p;	/* pointer to struct for target device		*/
{
    char    command[1200];
    char    font_target[1200];
    int	    rc,i,j;
    int     pixels;
	
/*  Compute pixel size, e.g. "300" in "cmr10.300pk"		*/
    pixels = (int) ((mag_value * p->density) + 0.5);
 
/*  Make sure that output font library exists and that we have 	*/
/*  write permission for it.					*/
    sprintf
	(command,
         "test -d %s",
         output_library); 
    rc = system(command);
    if_debug_show_rc;
    if (0 != rc) {
	fprintf
	    (stderr,
	     "Output directory \"%s\" does not exist.\n",
	     output_library);
	exit(1);
    }
    sprintf
	(command,
         "test -w %s",
         output_library);
    rc = system(command);
    if_debug_show_rc;
    if (0 != rc) {
	fprintf
	    (stderr,
	     "Can't write on output directory \"%s\".\n",
	     output_library);
	exit(1);
    }

/*  If writable subdirectory with name of device exists, use it	*/
/*  as the target for the font file; else use the output 	*/
/*  library itself.						*/
    strcpy(font_target,output_library);
    sprintf
	(command,
         "test -d %s/%s",
         output_library,
         p->dir_name);
    rc = system(command);
    if_debug_show_rc;
    if (0 == rc) {
	sprintf
	    (command,
	     "test -w %s/%s",
	     output_library,
	     p->dir_name);
	rc = system(command);
	if_debug_show_rc;
	if (0 == rc) {
	    sprintf
		(font_target,
	         "%s/%s",
	         output_library,
	         p->dir_name);
	}
    }
 
/*  See if the output font exists; skip invoking METAFONT if so	*/
    sprintf
	(command,
         "test -r %s/%s.%dpk",
         font_target,
         font_parm,
         pixels);
    rc = system(command);
    if_debug_show_rc;
    if (rc == 0) return;

/*  Run METAFONT						*/
    sprintf
	 (command,
	  "virmf \"&%s \\mode:=%s; %s nonstopmode; input %s\"",
	  base_parm, 
	  p->mode_name,
	  mag_string, 
	  font_parm);
    printf("    %s\n",command);
    rc = debug ? 0 : system(command);
	
/*  Run GFtoPK; "pk" font file goes to final position		*/
    sprintf
	(command,
         "gftopk %s.%dgf %s/%s.%dpk",
         font_parm,
         pixels,
         font_target,
         font_parm,
         pixels);
    printf("    %s\n",command);
    rc = debug ? 0 : system(command);
	
/*  Move the ".tfm" file to the output library			*/
    sprintf
	(command,
	 "mv -f %s.tfm %s/%s.tfm",
	 font_parm,
	 output_library, 
	 font_parm);
    printf("    %s\n",command);
    rc = debug ? 0 : system(command);

/*  If not "verbose" output, delete ".nnngf" and ".log" files	*/
	
    if (!verbose) {
	sprintf
	    (command,
	     "rm -f %s.log",
	     font_parm,
	     pixels,
	     font_parm);
	printf("    %s\n",command);
	rc = debug ? 0 : system(command);
    }
}

/****************************************************************/
/*								*/
/*  init() -- Process command-line parameters; make sure that 	*/
/*	      everything needed is present.			*/
/*								*/
/****************************************************************/

init(argc,argv)
int   argc;
char *argv[];
{
    int   i,error;

/*  Initialize the global variables				*/
    debug = verbose = FALSE;
    num_devices = 0;
    output_library = ".";
    font_parm = NULL;
    mag_parm = NULL;
    base_parm = "cm";

    error = FALSE;

/*  Look through the list of command line parameters		*/
    for (i=1; i<argc; i++) {
	if ('-' == argv[i][0]) {
	    switch (argv[i][1]) {
	      case 'b':
		if ('\0' == argv[i][2]) {
		    if (++i == argc) goto SYNTAX_ERROR;
		    base_parm = argv[i];
		}
		else base_parm = &argv[i][2];
		break;
	      case 'd':
		if ('\0' == argv[i][2]) {
		    if (++i == argc) goto SYNTAX_ERROR;
		    device_name[num_devices++] = argv[i];
		}
		else device_name[num_devices++] = &argv[i][2];
		break;
	      case 'l':
		if ('\0' == argv[i][2]) {
		    if (++i == argc) goto SYNTAX_ERROR;
		    output_library = argv[i];
		}
		else output_library = &argv[i][2];
		break;
	      case 'Z':
		debug = TRUE;
		break;
	      case 'v':
		verbose = TRUE;
		break;
	      default:
		fprintf
		    (stderr,
		     "Unrecognized flag \"%s\"\n",
		     argv[i]);
		error = TRUE;
		break;
	    }
	}
	else if (NULL == font_parm) {
	    font_parm = argv[i];
	}
	else if (NULL == mag_parm) {
	    mag_parm = argv[i];
	}
	else {
	    fprintf
		(stderr,
		 "Unrecognized parameter \"%s\"\n",
		 argv[i]);
	    error = TRUE;
	}
    }

/*  Check for required parameters				*/
    if (NULL == font_parm) error = TRUE;

/*  If no devices were specified, plug in a default list	*/
    if (0 == num_devices) {
	num_devices = 3;
	device_name[0] = "4216";
	device_name[1] = "3812";
	device_name[2] = "preview";
    }

/*  Return TRUE if initialization looks OK; FALSE otherwise.	*/
    if (!error) return(TRUE);

  SYNTAX_ERROR:
    fprintf
	(stderr,
	 "Usage:  makefont [-v] [-b base] [-d device]... [-l lib] font mag\n");
    return(FALSE);
}

/****************************************************************/
/*								*/
/*  lookup_device() -- Find a specified device's ctl block	*/
/*								*/
/****************************************************************/

device *lookup_device(str)
char   *str;
{
    device   *p = dev_anchor;

    while (NULL != p) {
	if (0 == strcmp(p->stanza_name,str))
	    return(p);
	p = p->next;
    }
    return(NULL);
}

/****************************************************************/
/*								*/
/*  check_mag() -- Validate requested magnification, fill in	*/ 
/*		   the "mag_string" and "mag_value" variables	*/
/*								*/
/****************************************************************/

check_mag()
{
    int    i;
    int    numerator,denominator;

/*  OK if no magnification specified				*/
    if (NULL == mag_parm) {	
	mag_string = "";
	mag_value = 1.0;
	return(TRUE);
    }

/*  Various "magstep" values are supported:			*/
    if (0 == strcmp(mag_parm,"magstep0")) {
	mag_string = "";
	mag_value = 1.0;
	return(TRUE);
    }
    if ((0 == strcmp(mag_parm,"magstep0.5")) ||
	(0 == strcmp(mag_parm,"magstephalf"))) {
	mag_string = "mag=magstep0.5;";
	mag_value = 1.095;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep1")) {
	mag_string = "mag=magstep1;";
	mag_value = 1.2;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep2")) {
	mag_string = "mag=magstep2;";
	mag_value = 1.44;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep3")) {
	mag_string = "mag=magstep3;";
	mag_value = 1.728;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep4")) {
	mag_string = "mag=magstep4;";
	mag_value = 2.074;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep5")) {
	mag_string = "mag=magstep5;";
	mag_value = 2.488;
	return(TRUE);
    }
/* "magstep" values above 5 are for SliTeX fonts */
    if (0 == strcmp(mag_parm,"magstep6")) {
	mag_string = "mag=2.986;";
	mag_value = 2.986;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep7")) {
	mag_string = "mag=3.583;";
	mag_value = 3.583;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep8")) {
	mag_string = "mag=4.300;";
	mag_value = 4.300;
	return(TRUE);
    }
    if (0 == strcmp(mag_parm,"magstep9")) {
	mag_string = "mag=5.160;";
	mag_value = 5.160;
	return(TRUE);
    }

/*  Not "magstep...".  Check for "nnnpt"			*/
    i = strlen(mag_parm);
    if ((i >= 3) && (0 == strcmp(mag_parm+i-2,"pt"))) {
	mag_parm[i-2] = '\0';
	numerator = atoi(mag_parm);
	for (i=strlen(font_parm)-1; i>0; i--) {
	    if (font_parm[i] < '0') break;
	    if (font_parm[i] > '9') break;
	}
	if ((font_parm[i] == '\0') ||
	   ((denominator = atoi(font_parm+i+1)) == 0) ||
	   (numerator == 0)) {
	    fprintf
		(stderr,
		 "Can't form magnification factor from \"%s\" and \"%s\".\n",
		 font_parm+i+1,mag_parm);
	    return(FALSE);
	}
	mag_string = (char *) malloc(32);
	sprintf(mag_string,"mag=%s/%s;",mag_parm,font_parm+i+1);
	mag_value = ((double) numerator) / ((double) denominator);
	return(TRUE);
    }
 
/*  Not "nnnpt".  Mag must be an arbitrary scale factor.	*/
    mag_value = -1.0;
    mag_value = atof(mag_parm);
    if (mag_value <= 0) {
	fprintf
	    (stderr,
	     "Can't decipher magnification factor \"%s\".\n",
	     mag_parm);
	return(FALSE);
    }
    mag_string = (char *) malloc(32);
    sprintf(mag_string,"mag=%f;",mag_value);
    return(TRUE);
}

