/****************************************************************/
/*								*/
/*   parse_devices() -- 					*/
/*		Locate the "devices" file, using $TEXFONTS,	*/
/*		read and parse the file, creating a chain	*/
/*		of "device" structures.  Return the TRUE if 	*/
/*		successful, FALSE otherwise.			*/
/*								*/
/****************************************************************/

#include <stdio.h>
#ifndef IBMRT
#include <stdlib.h>
#else
double atof();
#endif
#include "devices.h"

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

extern  device  *dev_anchor;

int parse_devices()
{
    char   *texfonts;
    char   command[1048];
    char   path[1030];
    char   *p,*q;
    int    l;
    FILE   *rc = NULL;
    device *d;

/*  Get value of environment variable "TEXFONTS"; 		*/
/*   supply default if none 					*/
    texfonts = (char *) getenv("TEXFONTS");
    if (NULL == texfonts) 
	texfonts = ".:/usr/lpp/tex/fonts";

/*  Locate the "devices" file in the font search path		*/
    p = q = texfonts;
    while (!rc) {
	while ((':' != *q) && ('\0' != *q)) q++;
	l = q - p;
	if (0 < l) {
	    strncpy(path,p,l);
	    strcpy(path+l,"/devices");
	    rc = freopen(path,"r",stdin);
	    if (rc) break;
	}
	if ('\0' == *q) break;
	q++;
	p = q;
    }
    if (!rc) return(FALSE);

/*  Read and parse the "devices" file				*/
    yyparse();

#ifdef DEBUG
    d = dev_anchor;
    while (NULL != d) {
	dump_struct(d);
	d = d->next;
    }
#endif /* DEBUG */
    return(TRUE);
}

/****************************************************************/
/*								*/
/*	yyerror() -- Parser's error message printer		*/
/*								*/
/****************************************************************/

yyerror(p)
char *p;
{
    printf("yyerror:  %s\n",p);
}

#ifdef DEBUG
/****************************************************************/
/*								*/
/*	dump_struct() -- Debugging dump of a device structure	*/
/*								*/
/****************************************************************/

dump_struct(p)
device *p;
{
    printf("\n  Device stanza for \"%s\":\n",p->stanza_name);
    printf("\tnext = 0x%X\n",p->next);
    printf("\tdensity = %g\n",p->density);
    printf("\tzoom_density = %g\n",p->zoom_density);
    printf("\taspect_ratio = %g\n",p->aspect_ratio);
    printf("\tdir_name = \"%s\"\n",p->dir_name);
    printf("\tzoom_dir_name = \"%s\"\n",p->zoom_dir_name);
    printf("\tmode_name = \"%s\"\n",p->mode_name);
    printf("\tpreview = \"%s\"\n",p->preview ? "TRUE" : "FALSE");
    printf("\tprotocol = %d\n",p->protocol);
}
#endif /* DEBUG */
